﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『模様選択用ComboBoxコントロール』
'  │・外見上は図柄しかないが、これはアイテムの描画で文字列を描いていないだけで、中では文字列で選択処理が行われている。
'  │・《ComboBox.Items》に、《Drawing2D.HatchStyle》列挙体から生成した名称一覧（配列）を登録する。
'  │・ComboBoxコントロールの描画をコードでコントロールするようにして、アイテムの名称を表示する代わりに、模様を描画する。《DrawMode》プロパティと《DrawItem》イベント。
'  │→こうすることで、アイテムは文字列で見た目は絵柄のComboBoxとなり、選択処理は通常通り行える。
'  │  実際に選択された《HatchStyle》値は、《[Enum].Parse(～)》で文字列（名称）から得られる。
'  │
'  │『線種選択』
'  │・選択する方法として、RadioButton と ComboBox のどちらが良いのか？ そもそも、破線系に魅力がなく、実線と点線の2択で十分なんだけど…。（ブラッシュアップの過程で実行ファイルが90KBを超えてしまったので、それまでケチっていた線種選択を実装することにした。）
'  │・RadioButton * 2 と 独自描画ComboBox で創り比べた結果、独自描画ComboBox の方が若干小さかった（実行ファイルで0.5KB程）。選択肢が増えて小さいなら断然 ComboBox を採用するでしょう。
'  │（線種選択用ComboBoxコントロールも、模様選択用ComboBoxコントロールと同じ方式）
'  └────────────





Public Class FormSetupConsole

    Private _Parameter As SettingParameter

    ''' <summary>
    ''' 設定値
    ''' </summary>
    ''' <returns></returns>
    Public Property Parameter As SettingParameter
        Get
            If _Parameter Is Nothing Then _Parameter = New SettingParameter
            Return _Parameter
        End Get
        Private Set(value As SettingParameter) '《Private》付きにすることで、外部から変更できなくする。
            _Parameter = value
        End Set
    End Property



    '===========================================================================================================



    ''' <summary>
    ''' 所有者と設定対象パラメータを指定してモーダルで表示する。
    ''' </summary>
    ''' <param name="Owner">所有者</param>
    ''' <param name="Parameter">対象の設定項目</param>
    ''' <returns></returns>
    Public Overloads Function ShowDialog(Owner As IWin32Window, Parameter As SettingParameter) As DialogResult

        Me.Parameter = Parameter
        Return MyClass.ShowDialog(Owner)

    End Function



    '===========================================================================================================



    Private Sub FormSetupConsole_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        '=== 初期設定 ===
        Me.Text &= My.Application.Info.Title
        Me.Icon = My.Resources.MainIcon

        '=== ヘルプ ===
        With hlpHint

            Dim temp As String = "ビューアの呼び出しに使用するキーの組み合わせを設定します。"
            .SetHelpString(chkAlt, temp & vbNewLine & "※[Alt]単独,[Alt]+[Shift]の組み合わせは、メニュー選択になるため使用できません。")
            .SetHelpString(chkCtrl, temp & vbNewLine & "※[Alt]は、必ず[Ctrl]と組み合わせてください。")
            .SetHelpString(chkShift, temp)

            temp = "チェックを入れると、ビューに"
            .SetHelpString(chkFoundation, temp & "下地（背景色と模様）が描かれ、透明ピクセルが見分け易くなります。")
            temp &= "{0}が表示されます。"
            .SetHelpString(chkSight, String.Format(temp, "照準の位置を示す枠"))
            .SetHelpString(chkLevel, String.Format(temp, "撮影場所からの縦横斜め方向を示す印"))
            .SetHelpString(chkGrid, String.Format(temp, "拡大されたピクセルを区切る線"))

            temp = "{0}の色を選択します。"
            .SetHelpString(btnSightColor, String.Format(temp, "枠"))
            .SetHelpString(btnLevelColor, String.Format(temp, "印"))
            .SetHelpString(btnGridColor, String.Format(temp, "線"))
            .SetHelpString(btnFoundationColor1, String.Format(temp, "背景"))
            .SetHelpString(btnFoundationColor2, String.Format(temp, "模様"))

            .SetHelpString(chkFoundationTexture, "模様を付けたい時に、チェックを入れます。")

            temp = "線種を選択します。"
            .SetHelpString(cmbGridLine, temp)
            .SetHelpString(cmbFoundationTexture, temp.Replace("線種", "模様"))

            temp = vbCrLf & "既定の書式から選んだり、お好みの書式を入力したりできます。" & vbCrLf & "各要素は、下枠を参照してください。"
            .SetHelpString(cmbFormat, "撮影場所の色（ARGB値）を取得する際の表現方法を選択します。" & temp & vbNewLine & "書式は、《String.Format》メソッド規準です。")
            .SetHelpString(cmbPattern, "色（ARGB値）を外部アプリケーションへ送信する際の表現方法を選択します。" & temp & vbNewLine & "書式は、《SendKeys.Send》メソッド規準です。")
            temp = "現在の入力内容を、既製リストに登録します。"
            .SetHelpString(btnFormatAdd, temp)
            .SetHelpString(btnPatternAdd, temp)
            temp = "現在の入力内容を、既製リストから削除します。"
            .SetHelpString(btnFormatRemove, temp)
            .SetHelpString(btnPatternRemove, temp)
            .SetHelpString(txtFormatHelp, "書式に使用する要素番号の案内です。")

            temp = "変更を破棄します。"
            .SetHelpString(btnCancel, temp)
            .SetHelpString(btnOK, temp.Replace("破棄", "確定") & vbNewLine & My.Application.Info.Title & " 終了時に保存されます。")

        End With

        '=== 模様一覧 ===
        cmbFoundationTexture.Items.AddRange(GetType(Drawing2D.HatchStyle).GetEnumNames)

        '=== 線種一覧 ===
        cmbGridLine.Items.AddRange(GetType(Drawing2D.DashStyle).GetEnumNames)
        cmbGridLine.Items.Remove(Drawing2D.DashStyle.Custom.ToString) '不都合な《Custom》を削除。

        '=== 現在値の反映 ===
        With Parameter
            With .Viewer
                With .Call
                    chkAlt.Checked = .AltKey
                    chkCtrl.Checked = .CtrlKey
                    chkShift.Checked = .ShiftKey

                    '起動時に呼び出しキー不正で設定コンソールが呼び出された場合の、キャンセルを防ぐ。
                    btnCancel.Enabled = Not DirectCast(Me.Owner, FormViewer_Main).CallKeyChecker(.AltKey, .CtrlKey, .ShiftKey, False)
                End With

                With .View
                    With .Sight
                        chkSight.Checked = .Enabled
                        btnSightColor.BackColor = .MainColor
                    End With

                    With .Level
                        chkLevel.Checked = .Enabled
                        btnLevelColor.BackColor = .MainColor
                    End With

                    With .Grid
                        chkGrid.Checked = .Enabled
                        btnGridColor.BackColor = .MainColor
                        cmbGridLine.Text = .LineStyle.ToString
                    End With

                    With .Foundation
                        chkFoundation.Checked = .Enabled
                        btnFoundationColor1.BackColor = .MainColor
                        btnFoundationColor2.BackColor = .SubColor
                        chkFoundationTexture.Checked = .Switch
                        cmbFoundationTexture.Text = .Texture.ToString
                    End With
                End With
            End With

            With .Command
                With .ColorFormat
                    If .Ready IsNot Nothing Then cmbFormat.Items.AddRange(.Ready)
                    cmbFormat.Text = .Using
                End With
                btnFormatAdd.Tag = cmbFormat
                btnFormatRemove.Tag = cmbFormat

                With .SendPattern
                    If .Ready IsNot Nothing Then cmbPattern.Items.AddRange(.Ready)
                    cmbPattern.Text = .Using
                End With
                btnPatternAdd.Tag = cmbPattern
                btnPatternRemove.Tag = cmbPattern
            End With
        End With

    End Sub

    Private Sub FormSetupConsole_FormClosing(sender As Object, e As FormClosingEventArgs) Handles Me.FormClosing

        '=== 不正チェック ===
        '起動時に呼び出しキー不正で設定コンソールが呼び出された場合の、キャンセルを防ぐ。
        If Not btnCancel.Enabled AndAlso Me.DialogResult = DialogResult.Cancel Then
            My.Computer.Audio.PlaySystemSound(Media.SystemSounds.Hand)
            e.Cancel = True
        End If

    End Sub

    Private Sub btnOK_Click(sender As Object, e As EventArgs) Handles btnOK.Click

        '=== 不正チェック ===
        If DirectCast(Me.Owner, FormViewer_Main).CallKeyChecker(chkAlt.Checked, chkCtrl.Checked, chkShift.Checked) Then
            Me.DialogResult = DialogResult.None
            Exit Sub
        End If

        '=== 設定値の更新 ===
        With Parameter
            With .Viewer
                With .Call
                    .AltKey = chkAlt.Checked
                    .CtrlKey = chkCtrl.Checked
                    .ShiftKey = chkShift.Checked
                End With

                With .View
                    With .Sight
                        .Enabled = chkSight.Checked
                        .MainColor = btnSightColor.BackColor
                    End With

                    With .Level
                        .Enabled = chkLevel.Checked
                        .MainColor = btnLevelColor.BackColor
                    End With

                    With .Grid
                        .Enabled = chkGrid.Checked
                        .MainColor = btnGridColor.BackColor
                        If cmbGridLine.SelectedIndex > -1 Then .LineStyle = DirectCast([Enum].Parse(GetType(Drawing2D.DashStyle), cmbGridLine.Text), Drawing2D.DashStyle) '未選択（《Drawing2D.DashStyle.Custom》が設定されている）時は、無視する。
                    End With

                    With .Foundation
                        .Enabled = chkFoundation.Checked
                        .MainColor = btnFoundationColor1.BackColor
                        .SubColor = btnFoundationColor2.BackColor
                        .Switch = chkFoundationTexture.Checked
                        .Texture = DirectCast([Enum].Parse(GetType(Drawing2D.HatchStyle), cmbFoundationTexture.Text), Drawing2D.HatchStyle)
                    End With
                End With
            End With

            With .Command
                With .ColorFormat
                    .Ready = ArrayList.Adapter(cmbFormat.Items).ToArray(GetType(String)) '《ComboBox.Items》コレクションの中身を一旦ArrayListクラスに変換し、それを文字列型配列に変換して代入する。
                    .Using = cmbFormat.Text
                End With

                With .SendPattern
                    .Ready = ArrayList.Adapter(cmbPattern.Items).ToArray(GetType(String)) '上に同じ。
                    .Using = cmbPattern.Text
                End With
            End With
        End With

    End Sub

    Private Sub chkAlt_CheckedChanged(sender As Object, e As EventArgs) Handles chkAlt.CheckedChanged

        '=== [Alt]には[Crtl]を ===
        'If chkAlt.Checked Then chkCtrl.Checked = True '勝手にチェックを付けると、混乱を招くかも。
        chkCtrl.Font = New Font(chkCtrl.Font, If(chkAlt.Checked, FontStyle.Bold, FontStyle.Regular))

    End Sub

    Private Sub tlpGuide_CellPaint(sender As Object, e As TableLayoutCellPaintEventArgs) Handles tlpAid.CellPaint

        '=== 区切り線を描画 ===
        If tlpAid.RowStyles(e.Row).SizeType = SizeType.Absolute Then
            e.Graphics.FillRectangle(New Drawing2D.HatchBrush(Drawing2D.HatchStyle.LightVertical, SystemColors.ControlDark, Color.Transparent), e.CellBounds)
        End If

        '関連項目毎に背景に色を付けても良かったけど、意外と手間だったので区切り線的に色を付けることにした。
        '手間な所 → CheckBoxの背景色と透明にしないといけない、項目を増やすと塗るマスも変わるなど、どのマスが関連しているのかコード上で記述しなければわからない。

    End Sub

    Private Sub btnColor_Click(sender As Object, e As EventArgs) Handles btnSightColor.Click, btnLevelColor.Click, btnGridColor.Click, btnFoundationColor1.Click, btnFoundationColor2.Click

        '=== 色選択 ===
        With clrColorPicker
            Dim target As Button = DirectCast(sender, Button) '対象
            .Color = target.BackColor
            If .ShowDialog = DialogResult.OK Then target.BackColor = .Color
        End With

    End Sub

    Private Sub cmbGridLine_DrawItem(sender As Object, e As DrawItemEventArgs) Handles cmbGridLine.DrawItem

        If e.Index < 0 Then Exit Sub '※※※ アイテムがないときは何もしない。

        '=== 線種アイテム描画 ===
        '--- 背景 ---
        e.DrawBackground()

        '--- 模様 ---
        Using p As Pen = New Pen(Color.Black) With {.DashStyle = DirectCast([Enum].Parse(GetType(Drawing2D.DashStyle), cmbGridLine.Items(e.Index).ToString), Drawing2D.DashStyle)}
            With e.Bounds
                Dim position As Integer = .Top + .Height \ 2
                e.Graphics.DrawLine(p, .Left, position, .Right, position)
                position = If(e.Index Mod 2 = 0, .Left + .Width \ 3, .Right - .Width \ 3)
                e.Graphics.DrawLine(p, position, .Top, position, .Bottom)
            End With
        End Using

        '--- フォーカス ---
        e.DrawFocusRectangle()

    End Sub

    Private Sub cmbFoundationTexture_DrawItem(sender As Object, e As DrawItemEventArgs) Handles cmbFoundationTexture.DrawItem

        If e.Index < 0 Then Exit Sub '※※※ アイテムがないときは何もしない。

        '=== 模様アイテム描画 ===
        '--- 背景 ---
        e.DrawBackground()

        '--- 模様 ---
        Dim box As Rectangle '形状

        With e.Bounds
            box = New Rectangle(.X + 4, .Y + 2, .Width - 8, .Height - 4)
        End With

        With e.Graphics
            .FillRectangle(New Drawing2D.HatchBrush(DirectCast([Enum].Parse(GetType(Drawing2D.HatchStyle), DirectCast(sender, ComboBox).Items(e.Index).ToString), Drawing2D.HatchStyle), e.ForeColor, e.BackColor), box)
            .DrawRectangle(Pens.Black, box)
        End With

        '--- フォーカス ---
        e.DrawFocusRectangle()

    End Sub

    Private Sub btnAdd_Click(sender As Object, e As EventArgs) Handles btnFormatAdd.Click, btnPatternAdd.Click

        '=== 既製リストに追加 ===
        With DirectCast(DirectCast(sender, Button).Tag, ComboBox)
            If Not .Items.Contains(.Text) Then .Items.Add(.Text)
        End With

    End Sub

    Private Sub btnRemove_Click(sender As Object, e As EventArgs) Handles btnFormatRemove.Click, btnPatternRemove.Click

        '=== 既製リストから削除 ===
        With DirectCast(DirectCast(sender, Button).Tag, ComboBox)
            Dim vestige As String = .Text
            .Items.Remove(.Text)
            .Text = vestige '削除しても、テキストだけは残す。
        End With

    End Sub

End Class