﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『ドラッグ中に使用する時の問題点』
'  │・ペイントソフト等で線を引くなどの操作をしているときには、当方のPictureBoxコントロールの《MouseMove》イベントが利用できない問題がある。
'  │→Timerコントロールで割り込み、スクリーン座標からPictureBoxコントロール座標に変換するしかない。
'  │・場合によっては、マウスカーソルも見えないかもしれない。
'  │→自分で描画するしかない。Button類も操作不能なので、picScreen以外（Formの枠も含めて）なくてもいいかも。
'  │
'  │『2つのTimerコントロール』
'  │・呼び出しキー監視とワープ先選択の実装には、1つのTimerコントロールで事足りるけど、2つ使っている。
'  │→その方が、プログラムが分かり易いから。インターバルを別々にできることで何かしら良いこともあるかもしれないし。
'  │　少なくとも、ビューア非表示時にワープ先選択タイマを停止することで、負荷が軽減するでしょう（っていうほど有意差はないでしょうが…）。
'  │
'  │『呼び出しキーの憂鬱』
'  │・呼び出しキーに何を使うか…“HELuPER!”からの難問の一つ。
'  │・本作は、その性格上アクティブになれないので、外部アプリケーションに直接作用するキーを使えない。そうなると、修飾キーしか選択肢がない。
'  │・しかも、修飾キーはそれそれ使用頻度が高いので、複数キーを組み合わせて使うのが望ましい。
'  │→これらを勘案すると、[Alt]+[Ctrl]+[Shift], [Alt]+[Ctrl], [Ctrl]+[Shift]の3つしか残らない。（[Alt]+[Shift]は、[Alt]単体と同じ認識になる場合がある。必ずではないようで不安定。）
'  │※もちろん、[Ctrl]や[Shift]を単体で使用しても、直ちに不都合があるわけではない。
'  │・特に[Alt]キーは、メニュー選択になる（メニューバーへフォーカスを移したり、メニュー選択のショートカットにつかられる）ので要注意。
'  │  →例えば、ビューアにコンテキストメニューを追加した場合、呼び出しキーとして[Alt]キーを押していると、そのメニューをクリックしても反応しなくなる。
'  │
'  │『《TrackBar》コントロールの自由度向上策』
'  │・通常、《TrackBar》コントロールは、長さ方向は調整できるが、幅は固定されている。しかし意外と幅広で余白が大きく見た目が悪い。
'  │→そこで、コンテナーコントロールに入れることで、見た目上、幅の調整を可能にしている。
'  │
'  │『コマンドボタンのコンテナー管理』
'  │・コマンドボタン数が増えると、個別に配置するのは手間なので、コンテナーコントロールで一体化させる。倍率調整用のコントロール類も同様。
'  │・使用するコンテナーコントロールは、単なる《Panel》でも問題ないけど、実行ファイルを若干小さくできる《FlowLayoutPanel》の方を使用している（0.5KB単位で表示されるエクスプローラでは確認できない場合もあるけど…）。自動整列で便利だし。
'  │
'  │『色記憶による色送信』
'  │・色送信に“記憶”という段階を設けているのは、色の取得元と送信先のウィンドウが異なる等でウィンドウの切り替えが必要な場合に対処するためというのが、表向きの理由（ビューアを移動できたら不要だけど…）。
'  │  一方、裏向きの理由は、R欄にフォーカスを持って行くのをどうしても忘れてしまう作者の間抜けさをカバーするため、フォーカス移動を送信手続き後半へ先延ばししたかったから（悲しすぎる理由…(;´д｀)トホホ）。
'  │・しかし、これではひと手間増えてしまうので、照準がアクティブ（ビュー内にマウスカーソルがある）ならそのピクセルの色を送信し、非アクティブなら記憶している色を送信する方法も試してみた。
'  │→この方式では、一つの作業に入口と出口が複数存在してしまい（操作の違いで送信される色が変わる、X状態）、便利さよりも複雑さが際立ってしまう。“シンプル”,“コンパクト”というコンセプトにそぐわない。
'  │・上記を勘案した結果、どんな場合にも対処できて、比較的説明文も書き易いであろうことから、一旦記憶するという回り道を選んだ。
'  └────────────





''' <summary>
''' ビューア（メインシステム）
''' </summary>
Public Class FormViewer_Main

    ''' <summary>キャンバス</summary>
    Private Structure Canvas

        ''' <summary>ビットマップ</summary>
        Public Bmp As Bitmap

        ''' <summary>グラフィックス</summary>
        Public Grp As Graphics

    End Structure



    '-----------------------------------------------------------------------------------------------------------



    ''' <summary>絶対座標表示用書式</summary>
    Private Const COORDINATE_FORMAT As String = "{0,5:D},{1,5:D}"

    ''' <summary>相対座標表示用書式</summary>
    Private Const OFFSET_FORMAT As String = COORDINATE_FORMAT & "({2,4:+##0;-##0;0},{3,4:+##0;-##0;0})"

    ''' <summary>ARGB(16)要素表示用書式</summary>
    Private Const ARGB16_FORMAT As String = "X2"

    ''' <summary>ARGB(10)要素表示用書式</summary>
    Private Const ARGB10_FORMAT As String = "D3"



    '-----------------------------------------------------------------------------------------------------------



    ''' <summary>設定内容</summary>
    Private SettingData As New SettingParameter

    ''' <summary>記録内容</summary>
    Private ResumeData As New ResumeParameter


    ''' <summary>タスクトレイ用アイコン</summary>
    Private TaskIcon As Canvas

    ''' <summary>撮影用キャンバス</summary>
    Private ViewScreen As Canvas

    ''' <summary>拡大画像用キャンバス</summary>
    Private MagnifyImageScreen As Canvas


    ''' <summary>基点（スクリーン座標系）</summary>
    Private CardinalPoint As Point

    ''' <summary>撮影点（スクリーン座標系）</summary>
    ''' <remarks>撮影場所=撮影点-基点…撮影場所は、基点から見た座標</remarks>
    Private ShootingPoint As Point = Point.Empty

    ''' <summary>照準位置（撮影画像内での絶対座標）</summary>
    Private SightLocation As Point

    ''' <summary>照準位置（撮影位置からの相対座標）</summary>
    Private SightPosition As Point


    ''' <summary>撮影範囲（ビューに写る範囲）</summary>
    Private ViewArea As Rectangle

    ''' <summary>撮影範囲の中心</summary>
    Private ViewCenter As Point


    ''' <summary>拡大画像の寸法</summary>
    Private MagnifyImageSize As Rectangle

    ''' <summary>拡大画像の表示ずらし量</summary>
    Private MagnifyImageOffset As Size


    ''' <summary>要保存フラグ</summary>
    Private SaveOrder As Boolean = False


    ''' <summary>ズームストッパの有効化フラグ</summary>
    Private ZoomStopper As Boolean = True

    ''' <summary>詳細表示モードの有効化フラグ</summary>
    Private DisplayMode As Boolean = False



    '===========================================================================================================



#Region "システム部"

    Protected Overrides ReadOnly Property CreateParams() As CreateParams

        Get
            '=== 非アクティブ化 ===
            'Microsoftによると、これだけでアクティブ化しないフォームになるみた。
            '《.ExStyle》プロパティは、“拡張ウィンドウスタイル”と呼ばれるもので、ウィンドウのいろいろな状態を設定できるみたい。
            '“拡張ウィンドウスタイル”で設定できる項目の内、「アクティブにならないようにする」ためのフラグが、《WS_EX_NOACTIVATE(&H8000000)》ということですな。ただし、Win2000やXP以降らしい。

            Const WS_EX_NOACTIVATE As Integer = &H8000000

            Dim p As CreateParams = MyBase.CreateParams
            If Not MyBase.DesignMode Then
                p.ExStyle = p.ExStyle Or (WS_EX_NOACTIVATE)
            End If
            Return p
        End Get

    End Property

    Private Sub FormMain_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        Dim commonText As String '共通文言

        '=== システムイベント関連付け ===
        AddHandler Microsoft.Win32.SystemEvents.SessionEnding, AddressOf SystemEvents_SessionEnding 'Windowsの終了イベントを捕捉する。

        '=== コマンドライン引数 ===
        Const b0 As Integer = 1, b1 As Integer = b0 << 1 'ビットマスク（《BitVector32.CreateMask》は定数にできないので計算で。《2 ^ n》より《1 << n》の方が軽いよね？）
        Dim loadOrder As Specialized.BitVector32 = New Specialized.BitVector32(b0 Or b1) '読込指示フラグ（bit0=設定 / bit1=記録）

        For Each command As String In My.Application.CommandLineArgs
            If command.Equals("/NONSTOP", StringComparison.OrdinalIgnoreCase) Then        'ズームストッパ-切
                ZoomStopper = False
            ElseIf command.Equals("/TEST", StringComparison.OrdinalIgnoreCase) Then       'テストモード-入
                tsmQuitNoOutput.Visible = True : DisplayMode = True
            ElseIf command.Equals("/RESET", StringComparison.OrdinalIgnoreCase) Then      'デフォルト化（ファイル毎に対応できるようになっているけど、必要性が低いのでコマンドとしては実装していない。）
                loadOrder.Item(b0) = False : loadOrder.Item(b1) = False : SaveOrder = True
            ElseIf command.StartsWith("/ZOOM=", StringComparison.OrdinalIgnoreCase) Then  '倍率調整倍率指定（整数のみなので、有用な値は限られ実用性低い。数値指定サンプルな存在。）
                Dim zoom As Integer
                If Integer.TryParse(command.Substring(command.IndexOf("="c) + 1), zoom) AndAlso zoom > 1 AndAlso zoom < 11 Then '条件式は左から順に評価されるようなので、こういう書き方も許される（たぶん）。
                    btnZoomIn.Tag = zoom : btnZoomOut.Tag = -zoom
                End If
            End If
        Next

        '=== ファイルの読み込み ===
        '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        commonText = "{1}ファイルの復号化に失敗したため、既定値で起動します。{0}{0}{2}"

        If loadOrder.Item(b0) Then
            Try
                XmlFileIO(GetSettingFilePath, SettingData)
            Catch ex As Exception
                MessageBox.Show(String.Format(commonText, vbNewLine, "設定", ex.Message), My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        End If

        If loadOrder.Item(b1) Then
            Try
                XmlFileIO(GetResumFilePath, ResumeData)
            Catch ex As Exception
                MessageBox.Show(String.Format(commonText, vbNewLine, "記録", ex.Message), My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        End If

        '=== ビューア初期化 ===
        Initialize()

        '=== ツールチップ文言登録 ===
        With ttpHint
            commonText = "ズーム {0}" & If(DisplayMode, " ({1}x)", "")
            .SetToolTip(btnZoomIn, String.Format(commonText, "イン", CInt(btnZoomIn.Tag)))
            .SetToolTip(btnZoomOut, String.Format(commonText, "アウト", "1/" & Math.Abs(CInt(btnZoomOut.Tag)).ToString))
            .SetToolTip(trkMagnification, "倍率調整" & If(DisplayMode, String.Format(" ({0:P0}～{1:P0})", trkMagnification.Minimum, trkMagnification.Maximum), ""))

            commonText = "ピクセル色 (A)"
            .SetToolTip(lblAlpha, commonText)
            .SetToolTip(lblRed, commonText.Replace("A"c, "R"c))
            .SetToolTip(lblGreen, commonText.Replace("A"c, "G"c))
            .SetToolTip(lblBlue, commonText.Replace("A"c, "B"c))
        End With

    End Sub

    ''' <summary>
    ''' 基本パラメータ設定
    ''' </summary>
    Private Sub Initialize()

        '=== 各調整幅の決定 ===
        '--- ピューア寸法 ---
        Me.MaximumSize = New Size(Screen.PrimaryScreen.WorkingArea.Width * 0.9, Screen.PrimaryScreen.WorkingArea.Height * 0.9) 'メイン画面作業領域の90%とする。

        '--- 倍率 ---
        With trkMagnification
            .Minimum = 4 '最小値は固定。
            .LargeChange = Math.Min(Me.MaximumSize.Width, Me.MaximumSize.Height) \ 125 'ビューア最大寸法の短辺を基準とする。
            .Maximum = .LargeChange * 8 '最小値から8クリックで到達できるように。
        End With

        '=== キャンバス作成 ===
        With ViewScreen
            .Bmp = New Bitmap(Me.MaximumSize.Width, Me.MaximumSize.Height)
            .Grp = Graphics.FromImage(.Bmp)
        End With

        With MagnifyImageScreen
            .Bmp = New Bitmap(Me.MaximumSize.Width, Me.MaximumSize.Height)
            .Grp = Graphics.FromImage(.Bmp)
            picView.Image = .Bmp
        End With

        '=== 設定値の適用 ===
        With SettingData.Viewer

            '--- タイマ関連 ---
            SaveOrder = SaveOrder Or SettingParameter.ValueTrimmer(.Call.ResponseInterval, 10, 5000) Or SettingParameter.ValueTrimmer(.View.ReplyInterval, 10, 1000) '有効範囲調整…上下限値は適当で、これを外れる数値はあまりにも無意味だから。修正後は、保存させる。
            tmrListen.Interval = .Call.ResponseInterval '呼び出しへの応答間隔
            tmrWatch.Interval = .View.ReplyInterval     '照準操作への応答間隔

            '--- 呼び出しキーの修正 ---
            With .Call
                If CallKeyChecker(.AltKey, .CtrlKey, .ShiftKey) Then tsmOption.PerformClick()
            End With

            '--- その他 ---
            SetupOther()

        End With

        '=== 記録値の適用 ===
        With ResumeData

            '--- ピューア寸法 ---
            SettingParameter.ValueTrimmer(.ViewerSize.Width, Me.MinimumSize.Width, Me.MaximumSize.Width)
            SettingParameter.ValueTrimmer(.ViewerSize.Height, Me.MinimumSize.Height, Me.MaximumSize.Height)
            Me.Size = .ViewerSize

            '--- 倍率 ---
            SettingParameter.ValueTrimmer(.Magnification, trkMagnification.Minimum, trkMagnification.Maximum)
            trkMagnification.Value = .Magnification

            '--- 基数 ---
            chkBaseSwitch.Checked = .BaseHex

            '--- 基点 ---
            CardinalPoint = .Cardinal

        End With

    End Sub

    ''' <summary>
    ''' 呼び出しキーの不正を判定する
    ''' </summary>
    ''' <param name="Alt">[Alt]キーの使用状況</param>
    ''' <param name="Ctrl">[Ctrl]キーの使用状況</param>
    ''' <param name="Shift">[Shift]キーの使用状況</param>
    ''' <returns>不正なら《True》を返す。</returns>
    Public Function CallKeyChecker(Alt As Boolean, Ctrl As Boolean, Shift As Boolean, Optional Message As Boolean = True) As Boolean

        '使用できない組み合わせ(0=不使用/1=使用)
        'Alt   0 1 1
        'Ctrl  0 0 0
        'Shift 0 0 1

        If (Not (Alt OrElse Ctrl OrElse Shift)) OrElse (Alt AndAlso (Not (Ctrl OrElse Shift))) OrElse (Alt AndAlso Shift AndAlso (Not Ctrl)) Then
            If Message Then MessageBox.Show("呼び出しキーを変更してください。" & vbNewLine & vbNewLine & "※1つ以上のキーが必要です。" & vbNewLine & "※[Alt]キーは、[Ctrl]キーとの併用が必須です。", My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return True
        Else
            Return False
        End If

    End Function

    ''' <summary>
    '''その他準備
    ''' </summary>
    Private Sub SetupOther()

        '=== ワープ先ラベルを照準色に合わせる ===
        With lblWarpSight
            .BackColor = SettingData.Viewer.View.Sight.MainColor
            .ForeColor = If(.BackColor.GetBrightness <= 0.5!, Color.White, Color.Black) '背景色の明るさを評価して、前景色を決定する。（HSBのブライトネス値を見て、暗ければ白に、明るければ黒に。）
        End With

        '=== コマンドボタン有無効 ===
        chkSendRGB.Enabled = (SettingData.Command.SendPattern.Using <> "")

        '=== 呼び出しキー通知 ===
        With SettingData.Viewer.Call
            Dim caption As String() = New String() {
                My.Application.Info.Title,
                "呼出：" & (If(.AltKey, "[Alt]-", String.Empty) & If(.CtrlKey, "-[Ctrl]-", String.Empty) & If(.ShiftKey, "-[Shift]", String.Empty)).Replace("--", "+").Replace("-", String.Empty)
            }
            ntfTask.Text = String.Join(vbNewLine, caption)
        End With

    End Sub

    Private Sub FormMain_Shown(sender As Object, e As EventArgs) Handles MyBase.Shown

        '=== 開始 ===
        Me.Visible = False
        TaskIcomMaker()
        ntfTask.Visible = True
        tmrListen.Start()

    End Sub

    Private Sub SystemEvents_SessionEnding(sender As Object, e As Microsoft.Win32.SessionEndingEventArgs)

        '=== OSからの終了要請 ===
        tsmQuit.PerformClick()

        '『常駐しながらWindowsが終了した場合の保存処理について』
        '手動で終了した時と同様に、《Me.Close》して発生する《FormClosing》イベントで行えば良いように思えるけど、この手順ではエラーで正常に終了できない。「俺の仕事終わりやのに何でファイルアクセスなんてするねん！！」ってこと？
        '仕方がないので、手動終了時と、OS終了時（この《Microsoft.Win32.SystemEvents.SessionEnding》イベント）の2か所で保存処理が必要になる。
        '今回は、[終了]メニューを呼び出すことで1か所に集約してみた。もしかしたら「なんでメニュー選択なんかするねん！」って怒られるかも。OSが終了する時にどういう挙動をしているのか、実際に目で見ることができないので、何の確証もないけど…。
        'OSさんも「終了して！」って言うんなら、もうちょっと（《FormClosed》イベントを）待ってくれてもいいのにね。
        '終了処理って、どういう手順が正しいんやろなぁ？

    End Sub

    Private Sub tsmQuit_Click(sender As Object, e As EventArgs) Handles tsmQuit.Click, tsmQuitNoOutput.Click

        '=== 保存 ===
        If Not tsmQuitNoOutput.Checked Then DataSaver() '《Sender Is tsmQuit》で事足りるけど、今後何か拡張して他所でも終了条件を確認したくなった時用のフラグとしてチェックを利用することにした。

        '=== 終了 ===
        Me.Close()

    End Sub

    ''' <summary>
    ''' 設定と記録ファイルを書き出す
    ''' </summary>
    Private Sub DataSaver()

        System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

        Dim commonText As String = "{1}ファイルの符号化に失敗したため、保存できませんでした。{0}{0}{2}" '共通文言

        '=== 設定 ===
        If SaveOrder Then
            Try
                XmlFileIO(SettingData, GetSettingFilePath)
            Catch ex As Exception
                MessageBox.Show(String.Format(commonText, vbNewLine, "設定", ex.Message), My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        End If

        '=== 記録 ===
        With ResumeData
            SaveOrder =
                .ViewerSize <> Me.Size OrElse
                .Magnification <> trkMagnification.Value OrElse
                .BaseHex <> chkBaseSwitch.Checked OrElse
                .Cardinal <> CardinalPoint '変更確認（設定値の変更確認は上記の保存処理で完了しているので、変数を使い回す。）

            If SaveOrder Then
                .ViewerSize = Me.Size
                .Magnification = trkMagnification.Value
                .BaseHex = chkBaseSwitch.Checked
                .Cardinal = CardinalPoint

                Try
                    XmlFileIO(ResumeData, GetResumFilePath)
                Catch ex As Exception
                    MessageBox.Show(String.Format(commonText, vbNewLine, "記録", ex.Message), My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
                End Try
            End If
        End With

    End Sub

    Private Sub FormMain_FormClosed(sender As Object, e As FormClosedEventArgs) Handles Me.FormClosed

        '=== システムイベント関連付け解除 ===
        RemoveHandler Microsoft.Win32.SystemEvents.SessionEnding, AddressOf SystemEvents_SessionEnding

        'タスクトレイ用アイコンは自動的に消してくれる（インスタンシングでFormを登録してくれている）みたいなので、手作業での削除処理を廃止した。

    End Sub

    Private Sub tsmAbout_Click(sender As Object, e As EventArgs) Handles tsmAbout.Click

        '=== バージョン情報 ===
        Static f As FormAbout = Nothing

        If f Is Nothing OrElse f.IsDisposed Then f = New FormAbout

        If f.Visible Then
            ShakeUpForm(f)
        Else
            f.Show()
        End If

    End Sub

    Private Sub tsmOption_Click(sender As Object, e As EventArgs) Handles tsmOption.Click

        '=== 設定コンソール ===
        Static f As FormSetupConsole = Nothing

        If f Is Nothing Then
            f = New FormSetupConsole

            Me.Location = CardinalPoint '表示スクリーン調整
            '子フォームを表示するとき、これを行っておかないと、前回ビューアを使用したスクリーンに表示されるため、見失う可能性も無きにしも非ず…なので実装してみた。
            '子フォームの《Locate》プロパティを算出するより、親フォーム（ビューア）を持ってきた方が手っ取り早い。
            '├ビューアの前回の表示位置なんて意味がない。
            '└子フォームの《StartPosition》プロパティも活かせる。

            If f.ShowDialog(Me, SettingData) = DialogResult.OK Then
                SetupOther()
                RefreshViewer()
                SaveOrder = True
            End If
            f.Dispose()
            f = Nothing
        Else
            ShakeUpForm(f)
        End If

    End Sub

    ''' <summary>
    ''' フォームを揺さぶる
    ''' </summary>
    ''' <param name="Target">対象フォーム</param>
    ''' <param name="Time">回数と振幅</param>
    Private Sub ShakeUpForm(ByRef Target As Form, Optional Time As Integer = 15)

        Target.Activate()
        Media.SystemSounds.Asterisk.Play()
        For i As Integer = Time To 0 Step -1
            Target.Left += If(i Mod 2 = 0, i, -i)
            Application.DoEvents()
            Threading.Thread.Sleep(25)
        Next

    End Sub

    Private Sub ntfTask_MouseClick(sender As Object, e As MouseEventArgs) Handles ntfTask.MouseClick

        '=== 一時停止 ===
        If e.Button = MouseButtons.Left Then tsmPause.PerformClick()

    End Sub

    Private Sub tsmPause_Click(sender As Object, e As EventArgs) Handles tsmPause.Click

        '=== 応答停止 ===
        If Me.Visible Then
            ntfTask.ShowBalloonTip(3000, My.Application.Info.Title, "ビューアの呼出中には、停止できません。", ToolTipIcon.Info)
        Else
            tmrListen.Enabled = tsmPause.Checked
            tsmPause.Checked = Not tsmPause.Checked
            TaskIcomMaker()
        End If

    End Sub

    ''' <summary>
    ''' タスクトレイ用アイコンを作成
    ''' </summary>
    Private Sub TaskIcomMaker()

        With TaskIcon

            '=== アイコン作成 ===
            If tsmPause.Checked Then
                '一時停止状態に合わせて暗転
                .Grp.FillRectangle(New SolidBrush(Color.FromArgb(&H80, Color.Black)), .Bmp.GetBounds(GraphicsUnit.Pixel))
            Else
                '通常のアイコン
                If .Bmp IsNot Nothing Then
                    .Bmp.Dispose()
                    .Grp.Dispose()
                End If
                .Bmp = New Icon(My.Resources.MainIcon, 16, 16).ToBitmap
                .Grp = Graphics.FromImage(.Bmp)
            End If

            '=== 画像置換 ===
            ntfTask.Icon = System.Drawing.Icon.FromHandle(.Bmp.GetHicon)

        End With

    End Sub

#End Region


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


#Region "ファイルI/O部"

    ''' <summary>
    ''' XMLファイルからの復元
    ''' </summary>
    ''' <param name="FilePath">ファイルのパス</param>
    ''' <param name="LoadObject">復元先オブジェクト</param>
    ''' <remarks></remarks>
    Private Sub XmlFileIO(FilePath As String, ByRef LoadObject As Object)

        '=== XMLファイルから読み込む ===
        If System.IO.File.Exists(FilePath) Then

            Try

                '--- XMLファイルから読み込み、逆シリアル化する ---
                Dim serializer As New System.Xml.Serialization.XmlSerializer(LoadObject.GetType) 'XMLシリアル化機構を作成する。
                Using streamer As New System.IO.FileStream(FilePath, System.IO.FileMode.Open) 'ファイルを開く。
                    LoadObject = CTypeDynamic(serializer.Deserialize(streamer), LoadObject.GetType) 'ファイルから復元する。
                End Using
                'Throw New Exception("テスト") 'テスト用にエラーを発生させてみる。

            Catch ex As Exception

                Throw 'エラーを再スロー

            End Try

        End If

    End Sub

    ''' <summary>
    ''' XMLファイルへの保存
    ''' </summary>
    ''' <param name="SaveObject">保存対象オブジェクト</param>
    ''' <param name="FilePath">ファイルのパス</param>
    ''' <remarks></remarks>
    Private Sub XmlFileIO(ByRef SaveObject As Object, FilePath As String)

        '=== XMLファイルに書き込む ===
        Try

            '--- シリアル化し、XMLファイルに保存する ---
            Dim serializer As New System.Xml.Serialization.XmlSerializer(SaveObject.GetType) 'XMLシリアル化機構を作成する。
            Using streamer As New System.IO.FileStream(FilePath, System.IO.FileMode.Create) 'ファイルを開く。
                serializer.Serialize(streamer, SaveObject) '用意したファイルに書き出す。
            End Using
            'Throw New Exception("テスト") 'テスト用にエラーを発生させてみる。

        Catch ex As Exception

            Throw 'エラーを再スロー

        End Try

    End Sub

    ''' <summary>
    ''' 設定ファイルパスを取得
    ''' </summary>
    ''' <returns>絶対パス</returns>
    Private Function GetSettingFilePath() As String

        Return System.IO.Path.GetFullPath(ConfigReader("Setting File Path", ".\Setting.xml"))

    End Function

    ''' <summary>
    ''' 記録ファイルパスを取得
    ''' </summary>
    ''' <returns>絶対パス</returns>
    Private Function GetResumFilePath() As String

        Return System.IO.Path.GetFullPath(ConfigReader("Resume File Path", ".\Resume.xml"))

    End Function

    ''' <summary>
    ''' アプリケーション構成ファイルから値を取得
    ''' </summary>
    ''' <param name="Key">キー</param>
    ''' <param name="Makeshift">仮値</param>
    ''' <returns></returns>
    Public Shared Function ConfigReader(Key As String, Makeshift As String) As String

        Dim value As String = System.Configuration.ConfigurationManager.AppSettings(Key) 'アプリケーション構成ファイルまたは指定キーが存在しない場合は、《Nothing》が返される。
        If value Is Nothing Then value = Makeshift
        Return value

    End Function

#End Region


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


#Region "ビューア制御部"

    Private Sub FormMain_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize

        '=== 寸法調整 ===
        If ntfTask.Visible AndAlso Me.Visible Then RefreshViewer()

    End Sub

    Private Sub btnZoom_Click(sender As Object, e As EventArgs) Handles btnZoomIn.Click, btnZoomOut.Click

        '=== ズーム イン/アウト ===
        With trkMagnification
            Dim zoom As Integer = CInt(DirectCast(sender, Button).Tag) '調整情報を取得。
            Dim newValue As Integer = If(Math.Sign(zoom) > 0, .Value * Math.Abs(zoom), .Value \ Math.Abs(zoom)) '新しい倍率（調整方向は符号で、調整倍率は絶対値で分かる。）
            SettingParameter.ValueTrimmer(newValue, .Minimum, .Maximum) 'ズーム ストッパ-切-時の上下制限。
            .Value = newValue
        End With

    End Sub

    Private Sub trkMagnification_ValueChanged(sender As Object, e As EventArgs) Handles trkMagnification.ValueChanged

        '=== 倍率変更 ===
        If ntfTask.Visible Then RefreshViewer() '起動時の初期値設定でこのイベントが呼び出されてしまうので、タスクトレイアイコンの有無で起動直後を判別する。

        '=== ズーム ストッパ ===
        If ZoomStopper Then
            With trkMagnification
                btnZoomIn.Enabled = (.Value * CInt(btnZoomIn.Tag) <= .Maximum)
                btnZoomOut.Enabled = (.Value \ Math.Abs(CInt(btnZoomOut.Tag)) >= .Minimum)
            End With
        End If

        'ズーム ストッパとは…
        '倍率が、現在値の倍数にならないのを防ぎます。
        '例えば、現在値20、最大値30 だったとします。ズーム インして2倍したとすると、最大値を超えてしまうので 30 になります。続いて、ズーム アウトして元の値に戻そうとしても、30の1/2で15になってしまいます。
        'このように、ズーム イン/アウトで元の倍率に戻らないのが、ちょっと嫌だったのでこのような機能を設けました。
        '最大・最小値まで使い切りたい場合は、TrakBarコントロールのつまみをドラッグか、クリックによるラージチェンジで可能。また、起動オプションでストッパを解除できるようにもした。

    End Sub

    Private Sub chkBaseSwitch_CheckedChanged(sender As Object, e As EventArgs) Handles chkBaseSwitch.CheckedChanged

        '=== 基数スイッチ切り替え ===
        chkBaseSwitch.Text = If(chkBaseSwitch.Checked, "HEX(16)", "DEC(10)")

    End Sub

    Private Sub tmrListen_Tick(sender As Object, e As EventArgs) Handles tmrListen.Tick

        '=== 呼び出しキー監視 ===
        '※要注意：例えば、ビューアにコンテキストメニューを追加した場合、呼び出しキーとして[Alt]キーを押していると、そのメニューをクリックしても反応しなくなる。
        If My.Computer.Keyboard.AltKeyDown = SettingData.Viewer.Call.AltKey AndAlso My.Computer.Keyboard.CtrlKeyDown = SettingData.Viewer.Call.CtrlKey AndAlso My.Computer.Keyboard.ShiftKeyDown = SettingData.Viewer.Call.ShiftKey Then

            If Not Me.Visible Then

                '--- 拡大画像更新 ---
                ShootingPoint = System.Windows.Forms.Cursor.Position 'マウスカーソルの現在位置を取得
                RefreshViewer()

                '--- ビューア出現 ---
                Me.Location += ShootingPoint - picView.PointToScreen(New Point(picView.Width \ 2, picView.Height \ 2)) '拡大像の中心をマウスカーソルに合わせる。

                '画面からのはみ出し対処
                Dim adjust As Size = Size.Empty '位置調整量
                Dim usingArea As Rectangle = Screen.GetWorkingArea(ShootingPoint) '表示許容範囲
                If Me.Left < usingArea.Left Then
                    adjust.Width = usingArea.Left - Me.Left
                ElseIf Me.Right > usingArea.Right Then
                    adjust.Width = usingArea.Right - Me.Right
                End If
                If Me.Top < usingArea.Top Then
                    adjust.Height = usingArea.Top - Me.Top
                ElseIf Me.Bottom > usingArea.Bottom Then
                    adjust.Height = usingArea.Bottom - Me.Bottom
                End If
                If adjust <> Size.Empty Then
                    Me.Location += adjust 'フォームをずらす。
                    System.Windows.Forms.Cursor.Position = ShootingPoint + adjust 'マウスカーソルをずらす。
                End If

                Me.Visible = True
                tmrWatch.Start()

            End If

        ElseIf Me.Visible Then

            '--- ビューア雲隠れ ---
            tmrWatch.Stop()

            'ワープ！！！
            System.Windows.Forms.Cursor.Position = ShootingPoint + SightPosition

            'RGB送信
            If chkSendRGB.Checked Then
                chkSendRGB.Checked = False
                SendRGB()
            End If

            Me.Visible = False

        End If

    End Sub

    Private Sub tmrWatch_Tick(sender As Object, e As EventArgs) Handles tmrWatch.Tick

        '=== 照準移動 ===
        Dim insidePosition As Point = picView.PointToClient(MousePosition) '拡大画像内でのマウスカーソル位置
        Dim active As Boolean = picView.ClientRectangle.Contains(insidePosition) '照準のアクティブフラグ（マウスカーソルがビュー内に存在するか？）

        If active Then
            insidePosition -= MagnifyImageOffset
            SightLocation.X = insidePosition.X \ trkMagnification.Value
            SightLocation.Y = insidePosition.Y \ trkMagnification.Value
            SightPosition = SightLocation - ViewCenter
        Else
            SightLocation = ViewCenter
            SightPosition = Point.Empty
        End If

        '--- ビュー更新 ---
        picView.Refresh() '《Paint》イベントを呼び出す。

        '--- 座標表示 ---
        lblWarpSight.Text = String.Format(OFFSET_FORMAT,
                                          ShootingPoint.X - CardinalPoint.X + SightPosition.X,
                                          ShootingPoint.Y - CardinalPoint.Y + SightPosition.Y,
                                          SightPosition.X,
                                          SightPosition.Y)

    End Sub

    Private Sub picView_MouseClick(sender As Object, e As MouseEventArgs) Handles picView.MouseClick, picView.MouseDoubleClick
        '連続でクリックされた場合にも対処せるため、《MouseDoubleClick》イベントも併記する。
        '単なる《Click》イベントなら、連続でクリックした場合でも反応するみたい。
        'しかし、右クリックを使用した機能拡張の余地を残すために、それを判別できない《Click》イベントは使えない。

        Select Case e.Button
            Case MouseButtons.Left

                '=== 撮影場所を照準位置へ移動 ===
                ShootingPoint += SightPosition
                RefreshViewer()

            Case MouseButtons.Middle

                '=== 基点を照準位置へ移動 ===
                btnSetCardinal.PerformClick()

        End Select

    End Sub

    ''' <summary>
    ''' ビューア更新
    ''' </summary>
    Private Sub RefreshViewer()

        '=== 拡大画像作成 ===
        SetViewParameter()
        If ImageCapture() Then
            DrawBlueScreen()
        Else
            DrawViewImage()
        End If
        picView.Refresh()

        '=== 座標表示 ===
        lblCardinal.Text = String.Format(COORDINATE_FORMAT, CardinalPoint.X, CardinalPoint.Y)                                     '基点
        lblShooting.Text = String.Format(COORDINATE_FORMAT, ShootingPoint.X - CardinalPoint.X, ShootingPoint.Y - CardinalPoint.Y) '撮影場所（基点から見た相対位置）

    End Sub

    ''' <summary>
    ''' 拡大パラメータ作成
    ''' </summary>
    Private Sub SetViewParameter()

        With ViewArea
            .Width = Math.Ceiling(picView.ClientSize.Width / trkMagnification.Value)   '┬ 拡大後、ビューに写り込む範囲（要するに、撮影範囲）を求める。
            .Height = Math.Ceiling(picView.ClientSize.Height / trkMagnification.Value) '│
            .Width += 1 - (.Width Mod 2)   '┬ 撮影場所が中心に来るように奇数化する。   │
            .Height += 1 - (.Height Mod 2) '┘                                          ┘

            ViewCenter.X = .Width \ 2  '┬ 撮影範囲の中心を求める。
            ViewCenter.Y = .Height \ 2 '┘

            .Location = ShootingPoint - ViewCenter '撮影範囲の左上座標を求める。

            MagnifyImageSize = New Rectangle(0, 0, .Width * trkMagnification.Value, .Height * trkMagnification.Value) '拡大した画像の寸法を求める。
            MagnifyImageOffset = New Size((picView.ClientSize.Width - MagnifyImageSize.Width) \ 2, (picView.ClientSize.Height - MagnifyImageSize.Height) \ 2) '拡大した画像をビューに表示した時にはみ出す量を求める。
        End With

    End Sub

    ''' <summary>
    ''' 対象領域を撮影する
    ''' </summary>
    ''' <return>警告フラグ(キャプチャ失敗=[True]）</return>
    Private Function ImageCapture() As Boolean

        Dim alert As Boolean = False

        With ViewScreen.Grp

            .Clear(Color.Transparent)
            '《Graphics.CompositingMode》プロパティに《Drawing2D.CompositingMode.SourceCopy》を指定しても、
            '《Graphics.CopyFromScreen》の《copyPixelOperation》パラメータに《CopyPixelOperation.SourceCopy》を指定しても、
            'コピー元とコピー先が合成されてしまう(透明色をそのままコピーしてくれない)ので、クリアしておく。

            Me.Opacity = 0# 'ビューア寸法や倍率調整時に自分自身が写り込まないように透明にする。
            Try
                'Throw New System.Exception("テスト")
                .CopyFromScreen(ViewArea.Location, Point.Empty, ViewArea.Size) '←この1行だけでスクリーンショットできちゃう！！
            Catch
                alert = True 'スクリーンセーバーが走るとキャプチャに失敗するみたい。ただ、本作ではスクリーンセーバーの作動中にキャプチャすることはないので不要かも。
            End Try
            Me.Opacity = 1.0#

            Return alert

        End With

    End Function

    ''' <summary>
    ''' ブルースクリーンを描画
    ''' </summary>
    Private Sub DrawBlueScreen()

        With MagnifyImageScreen

            '=== 初期化 ===
            .Grp.ResetTransform()
            .Grp.Clear(Color.Blue)

            '=== キャプチャ失敗 ===
            .Grp.DrawString("キャプチャ失敗 (;_;)", Me.Font, New SolidBrush(Color.White), Point.Empty)

        End With

    End Sub

    ''' <summary>
    ''' 拡大画像を描画
    ''' </summary>
    Private Sub DrawViewImage()

        With MagnifyImageScreen

            '=== 共通変数 ===
            Dim sp As SettingParameter.ViewerParameter.ViewParameter = SettingData.Viewer.View '設定値へのアクセス簡便化用
            Dim x, y As Integer '座標ループ
            Dim bigPixel As Rectangle = New Rectangle(0, 0, trkMagnification.Value, trkMagnification.Value) '拡大ピクセル描画用

            '=== 初期化 ===
            .Grp.ResetTransform()
            .Grp.TranslateTransform(MagnifyImageOffset.Width, MagnifyImageOffset.Height)

            '=== 下地 ===
            If sp.Foundation.Enabled Then
                If sp.Foundation.Switch Then
                    .Grp.FillRectangle(New Drawing2D.HatchBrush(sp.Foundation.Texture, sp.Foundation.SubColor, sp.Foundation.MainColor), MagnifyImageSize)
                Else
                    .Grp.FillRectangle(New SolidBrush(sp.Foundation.MainColor), MagnifyImageSize)
                End If
            Else
                .Grp.Clear(Color.Transparent)
            End If

            '=== ピクセル ===
            Using pixelBrush As New SolidBrush(Color.Black) '拡大ピクセル描画用ブラシ（色はループ内で取得するので、ダミーで初期化する。）
                For y = 0 To ViewArea.Height - 1
                    For x = 0 To ViewArea.Width - 1
                        pixelBrush.Color = ViewScreen.Bmp.GetPixel(x, y)
                        .Grp.FillRectangle(pixelBrush, bigPixel)
                        bigPixel.X += trkMagnification.Value
                    Next
                    bigPixel.X = 0
                    bigPixel.Y += trkMagnification.Value
                Next
            End Using

            '=== 格子 ===
            If sp.Grid.Enabled Then
                Using drawPen As New Pen(sp.Grid.MainColor) With {.DashStyle = sp.Grid.LineStyle}
                    For x = 0 To MagnifyImageSize.Width - 1 Step trkMagnification.Value * 2
                        .Grp.DrawRectangle(drawPen, x, 0, trkMagnification.Value, MagnifyImageSize.Height)
                    Next
                    For y = trkMagnification.Value To MagnifyImageSize.Height - 1 Step trkMagnification.Value * 2
                        .Grp.DrawRectangle(drawPen, 0, y, MagnifyImageSize.Width, trkMagnification.Value)
                    Next
                End Using
            End If

            '=== 水準 ===
            If sp.Level.Enabled Then
                Using drawPen As New Pen(sp.Level.MainColor)
                    For i As Integer = 1 To 4 '上下左右の4方向且つ斜め4方向で、4回ループ。

                        '縦横用
                        x = ViewCenter.X * trkMagnification.Value + Choose(i, 0, trkMagnification.Value, 0, 0)
                        y = ViewCenter.Y * trkMagnification.Value + Choose(i, 0, 0, trkMagnification.Value, 0)
                        Do
                            .Grp.DrawLine(drawPen, x, y, x + If(i Mod 2 = 1, trkMagnification.Value, 0), y + If(i Mod 2 = 0, trkMagnification.Value, 0))
                            x += trkMagnification.Value * Choose(i, 0, +1, 0, -1)
                            y += trkMagnification.Value * Choose(i, -1, 0, +1, 0)
                        Loop While MagnifyImageSize.Contains(x, y)

                        '斜め用
                        x = ViewCenter.X * trkMagnification.Value + Choose(i, 0, trkMagnification.Value, trkMagnification.Value, 0)
                        y = ViewCenter.Y * trkMagnification.Value + Choose(i, 0, 0, trkMagnification.Value, trkMagnification.Value)
                        Do
                            .Grp.DrawLine(drawPen, x - 2, y, x + 2, y) '“━” ┬→ 合わせて “┼”
                            .Grp.DrawLine(drawPen, x, y - 2, x, y + 2) '“┃” ┘
                            x += trkMagnification.Value * Choose(i, -1, +1, +1, -1)
                            y += trkMagnification.Value * Choose(i, -1, -1, +1, +1)
                        Loop While MagnifyImageSize.Contains(x, y)

                    Next
                End Using
            End If

        End With

    End Sub

    Private Sub picView_Paint(sender As Object, e As PaintEventArgs) Handles picView.Paint

        '=== ビュー描画 ===
        With e.Graphics

            '--- 照準 ---
            Static animation As Integer = 0 'アニメーションのカウンタ
            If SettingData.Viewer.View.Sight.Enabled Then
                Using drawPen As Pen = New Pen(SettingData.Viewer.View.Sight.MainColor),
                      shadowPen As Pen = New Pen(Color.FromArgb(&H80, lblWarpSight.ForeColor), 3) 'ペン

                    Dim square As Rectangle = New Rectangle(SightLocation.X * trkMagnification.Value, SightLocation.Y * trkMagnification.Value, trkMagnification.Value, trkMagnification.Value) '照準を成す四角形

                    .TranslateTransform(MagnifyImageOffset.Width, MagnifyImageOffset.Height)

                    square.Inflate(-1, -1)
                    .DrawRectangle(drawPen, square)

                    animation += If(animation > 2, -1, 3)
                    square.Inflate(animation, animation)
                    .DrawRectangle(shadowPen, square)
                    .DrawRectangle(drawPen, square)

                End Using
            End If

            '--- 詳細表示モード用パラメータ ---
            If DisplayMode AndAlso SightPosition.IsEmpty Then '非表示条件は、『マウスカーソルがビュー内にある時』はとしたかったけど、座標計算と描画処理が分離されている都合上ここでそれを知れないので、代替え案として『照準が撮影場所にない時』とした。
                Using grpPath As New Drawing2D.GraphicsPath, sFormat As New StringFormat With {.Alignment = StringAlignment.Far, .LineAlignment = StringAlignment.Far}
                    grpPath.AddString(String.Format("W{1}{0}H{2}", vbNewLine, ViewArea.Width, ViewArea.Height), Me.Font.FontFamily, FontStyle.Bold, 30, New Point(0, 0), StringFormat.GenericDefault) '撮影範囲
                    grpPath.AddString(trkMagnification.Value.ToString("P0"), Me.Font.FontFamily, FontStyle.Bold, 30, New Point(picView.ClientRectangle.Right, picView.ClientRectangle.Bottom), sFormat) '倍率（桁区切りを止めるのは手間なのでそのまま）
                    .SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
                    .ResetTransform()
                    .FillPath(Brushes.White, grpPath)
                    .DrawPath(Pens.Black, grpPath)
                End Using
            End If

        End With

        '=== ピクセル色表示 ===
        Dim format As String = If(chkBaseSwitch.Checked, ARGB16_FORMAT, ARGB10_FORMAT) '書式
        With ViewScreen.Bmp.GetPixel(SightLocation.X, SightLocation.Y)
            lblAlpha.Text = .A.ToString(format)
            lblRed.Text = .R.ToString(format)
            lblGreen.Text = .G.ToString(format)
            lblBlue.Text = .B.ToString(format)
        End With

    End Sub

#End Region


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


#Region "コマンド部"

    Private Sub btnSetCardinal_Click(sender As Object, e As EventArgs) Handles btnSetCardinal.Click

        '=== 基点を照準位置へ移動 ===
        CardinalPoint = ShootingPoint + SightPosition
        RefreshViewer()

    End Sub

    Private Sub btnResetCardinal_Click(sender As Object, e As EventArgs) Handles btnResetCardinal.Click

        '=== 基点を原点に戻す ===
        CardinalPoint = Screen.PrimaryScreen.WorkingArea.Location
        RefreshViewer()

    End Sub

    Private Sub btnGotoCardinal_Click(sender As Object, e As EventArgs) Handles btnGotoCardinal.Click

        '=== 撮影場所を基点へ移動 ===
        ShootingPoint = CardinalPoint
        RefreshViewer()

    End Sub

    Private Sub btnCopyColor_Click(sender As Object, e As EventArgs) Handles btnCopyColor.Click

        Static once As Boolean = True 'バルーンチップを1回だけ表示するためのフラグ

        '=== 送信用に記憶 ===
        btnCopyColor.BackColor = ViewScreen.Bmp.GetPixel(ViewCenter.X, ViewCenter.Y) '技術要綱に詳記有り。

        '=== カラーコードをクリップボードへコピーする ===
        Try

            Clipboard.SetText(GetColorString(btnCopyColor.BackColor, SettingData.Command.ColorFormat.Using))
            once = True

        Catch ex As ArgumentNullException

            If once Then
                ntfTask.ShowBalloonTip(3000, My.Application.Info.Title, "書式が未設定です。" & vbNewLine & "設定されるまで、記憶のみ行います。", ToolTipIcon.Info)
                once = False
            End If
            'なぜか、《GetColorString》内の《String.Format》の例外がこっちで発生する。謎？？
            'もしかして、コンパイラの最適化の結果、Fanctionの中身がここに直接書かれているのと同じことになってしまうのかな？

        Catch ex As Exception

            MessageBox.Show("カラーコードのクリップボードへのコピーに失敗しました。" & vbCrLf & vbCrLf & ex.Message, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try

    End Sub

    ''' <summary>
    ''' R,G,B値を外部へ送信する
    ''' </summary>
    Private Sub SendRGB()

        Try

            '=== キーストロークのパターン整形 ===
            'パターンは以下の2種類に対応できると便利であるが、どちらも最終的に《String.Format》メソッドに通さなければならないので、それに適した形に変換する必要がある。
            '・《String.Format》メソッド用に記述した直接的パターン                       例"^a{4}{{TAB}}^a{5}{{TAB}}^a{6}"    →  多様な数値出力が可能な代わりにキーストロークの記述が面倒になる（波括弧を二重にしなければならない）。
            '・簡易置換マーカーを使用して《SendKeys》メソッド用に記述した間接的パターン  例"^a|R10|{TAB}^a|G10|{TAB}^a|B10|"  →  数値を10進数RGBに固定化する代わりにキーストローク部分は素直に書ける。
            'そこで、正規表現の力を借りて間接的パターンを直接的パターンに変換する。

            Dim pattern As String = System.Text.RegularExpressions.Regex.Replace(SettingData.Command.SendPattern.Using, "(?<!{)\{(?<key>[^0-9{].*?)\}", "{{${key}}}") '"{キー名称}"（一重波括弧のみ）を"{{キー名称}}"（二重波括弧）に変換する。
            '引数2《pattern》     = 「前に始波括弧がない始波括弧 "(?<!{)\{"」「"key"名でグループ化 "(?<key>～)"」「数字と始波括弧以外で始まって "[^0-9{]"」「終波括弧で終わる最短マッチの文字列 "[^0-9{].*?\}"」
            '引数3《replacement》 = 「"key"名でグループ化した文字列を二重波括弧で挟む "{{${key}}}"」

#If DEBUG Then
            Console.Write(pattern) 'デバッグ：整形したパターンを出力する。
#End If

            '=== パターン生成 ===
            pattern = GetColorString(btnCopyColor.BackColor, pattern)

#If DEBUG Then
            Console.WriteLine(" => " & pattern) 'デバッグ：生成したパターンを出力する。
#End If

            '=== 送信 ===
            SendKeys.Send(pattern) '.NETの《[SendKeys.Send》メソッドは、カレット（"^"）を送信できない不具合がある。
            'CreateObject("WScript.Shell").SendKeys(pattern) 'WSH側の《.SendKeys》なら送れる。どちらを使用するべきなのか・・。

        Catch ex As Exception

            MessageBox.Show("送信パターンの生成と送信に失敗しました。" & vbCrLf & vbCrLf & ex.Message, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try

    End Sub

    ''' <summary>
    ''' 色を色書式で表現する
    ''' </summary>
    ''' <param name="Target">色</param>
    ''' <param name="Format">色書式</param>
    ''' <returns>指定書式で表現された色コード文字列</returns>
    Public Function GetColorString(Target As Color, Format As String) As String

        Try

            Dim Values As Object() 'わざわざ配列にする必要はないけど、ここは可読性を優先してみた。
            With Target
                Values = New Object() {
                    vbNewLine,
                    .ToArgb,
                    .ToArgb And &HFFFFFF,
                    .A,
                    .R,
                    .G,
                    .B,
                    CDbl(.A / 255)
                } '{0}:改行, {1}:ARGB, {2}:RGB(A値なし), {3}:A, {4}:R, {5}:G, {6}:B, {7}:A(0.0F～1.0F)
            End With

            Format = Format.Replace(vbCr, "").Replace(vbLf, vbCrLf) 'UTF-8（設定ファイルの文字コード）での改行コード"LF"を、Windowsの"CRLF"に置換する。
            Format = Format.Replace("|A10|", "{3}").Replace("|R10|", "{4}").Replace("|G10|", "{5}").Replace("|B10|", "{6}")             '┬ 簡易置換マーカーを清書する。
            Format = Format.Replace("|A16|", "{3:X2}").Replace("|R16|", "{4:X2}").Replace("|G16|", "{5:X2}").Replace("|B16|", "{6:X2}") '┘

            'Throw New Exception("テスト") 'テスト用にエラーを発生させてみる。
            Return String.Format(Format, Values) '例外《ArgumentNullException》は、ここで《Catch》できず、呼び出し元で発生するみたい。なんで(?_?)

        Catch ex As Exception

            Throw New ApplicationException("色の書式表現に失敗しました。" & vbCrLf & "（" & ex.Message & "）") '呼び出し元で例外を発生させる。《ApplicationException》なのは、何となく…。

        End Try

    End Function

    Private Sub lblARGB_Click(sender As Object, e As EventArgs) Handles lblAlpha.Click, lblRed.Click, lblGreen.Click, lblBlue.Click

        '=== 色要素をクリップボードへコピーする ===
        'おまけ機能…あまり必要性を感じないけど、見易さを優先して DEC/HEX 表示を分離したのでこういうことがし易くなったから実装してみただけ。

        Try
            Clipboard.SetText(DirectCast(sender, Label).Text)
        Catch ex As Exception
            MessageBox.Show("クリップボードへの色要素コピーに失敗しました。" & vbCrLf & vbCrLf & ex.Message, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

#End Region

End Class
