﻿''' <summary>
''' 設定項目
''' </summary>
''' <remarks></remarks>
<Serializable()>
<System.Xml.Serialization.XmlRoot("SCOrPioの設定内容")>
Public Class SettingParameter

    ''' <summary>
    ''' 有効範囲内に値を収める
    ''' </summary>
    ''' <param name="Value">調整する値</param>
    ''' <param name="Low">最小値</param>
    ''' <param name="Hi">最大値</param>
    ''' <return>変更があったら[True]を返す</return>
    Public Shared Function ValueTrimmer(ByRef Value As Integer, Low As Integer, Hi As Integer) As Boolean

        Dim temp As Integer = Value
        Value = Math.Max(Low, Math.Min(Value, Hi))

        Return (Value <> temp)

    End Function



    '============================================================================================================================



    ''' <summary>
    ''' ビューアに関する設定
    ''' </summary>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("ビューア")>
    Public Property Viewer() As ViewerParameter = New ViewerParameter

    ''' <summary>
    ''' コマンドに関する設定
    ''' </summary>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("コマンド")>
    Public Property Command() As CommandParameter = New CommandParameter 'ビューア内に置くべきか迷う。



    '============================================================================================================================



    ''' <summary>
    ''' ビューアに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()>
    Public Class ViewerParameter

        ''' <summary>
        ''' 呼び出しに関する設定
        ''' </summary>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("呼出")>
        Public Property [Call]() As CallParameter = New CallParameter

        ''' <summary>
        ''' ビューに関する設定
        ''' </summary>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ビュー")>
        Public Property View() As ViewParameter = New ViewParameter



        '============================================================================================================================



        ''' <summary>
        ''' 呼び出しに関する設定項目
        ''' </summary>
        <Serializable>
        Public Class CallParameter

            ''' <summary>
            ''' 応答間隔を設定・取得します
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("応答間隔")>
            Public Property ResponseInterval() As Integer = 100

            ''' <summary>
            ''' [Alt]キー使用フラグ
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("Alt")>
            Public Property AltKey As Boolean = True

            ''' <summary>
            ''' [Ctrl]キー使用フラグ
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("Ctrl")>
            Public Property CtrlKey As Boolean = True

            ''' <summary>
            ''' [Shift]キー使用フラグ
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("Shift")>
            Public Property ShiftKey As Boolean = False

        End Class



        '============================================================================================================================



        ''' <summary>
        ''' ビューに関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()>
        Public Class ViewParameter

            ''' <summary>
            ''' 応答間隔を設定・取得します
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("応答間隔")>
            Public Property ReplyInterval() As Integer = 50

            ''' <summary>
            ''' 照準
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("照準")>
            Public Property Sight() As Switch1Color1 = New Switch1Color1 With {.MainColor = Color.OrangeRed}

            ''' <summary>
            ''' 水準
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("水準")>
            Public Property Level() As Switch1Color1 = New Switch1Color1 With {.MainColor = Color.Orange}

            ''' <summary>
            ''' 格子
            ''' </summary>
            ''' <remarks>直線に比べて、点線は控えめで見易く気に入った。反対に破線系は、見難く絵面が汚い。</remarks>
            <System.Xml.Serialization.XmlElement("格子")>
            Public Property Grid() As Switch1Color1Style1 = New Switch1Color1Style1 With {.MainColor = Color.Black, .LineStyle = Drawing2D.DashStyle.Dot}

            ''' <summary>
            ''' 下地
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("下地")>
            Public Property Foundation() As Switch2Color2Texture1 = New Switch2Color2Texture1 With {.MainColor = Color.Black, .SubColor = Color.Gray, .Switch = True, .Texture = Drawing2D.HatchStyle.BackwardDiagonal}



            '============================================================================================================================



            'このクラスをシリアル化する上での最大の難点は、Color構造体にある。
            'Color（Fontも）は、シリアライズできないからだ。
            '様々な実装方法を試行錯誤するも、なかなか簡単で綺麗な解決法が見つからない。宇使う色数が増えれば増えるほど難儀になる。

            ''' <summary>
            ''' スイッチ付きの色
            ''' </summary>
            ''' <remarks>1スイッチ/1色</remarks>
            <Serializable()>
            Public Class Switch1Color1

                ''' <summary>
                ''' 有効/無効
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("有効")>
                Public Property Enabled() As Boolean = True

                ''' <summary>
                ''' 主色
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks>《Color》構造体を直接シリアライズできないので出力対象外とし、代わりに《MainColorCode》を用意する。</remarks>
                <System.Xml.Serialization.XmlIgnoreAttribute()>
                Public Property MainColor() As Color = Color.White

                ''' <summary>
                ''' 主色のコード
                ''' </summary>
                ''' <value>色名（大文字小文字不問）または16進数ARGB値（"0x"付きも可）</value>
                ''' <returns>色名またはARGB値（16進数）</returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("主色")>
                <System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never), System.ComponentModel.Browsable(False)>
                Public Property MainColorCode() As String

                    Get
                        Return ColorExport(MainColor)
                    End Get

                    Set(ByVal value As String)
                        MainColor = ColorImport(value)
                    End Set

                End Property



                '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



                ''' <summary>
                ''' 色を文字列に変換する
                ''' </summary>
                ''' <param name="Value">《Color》構造体</param>
                ''' <returns>文字列</returns>
                ''' <remarks></remarks>
                Protected Function ColorExport(Value As Color) As String

                    Return Value.Name '名前がある場合は色名が、ない場合はARGB値（16進数）が返される。便利。
                    'Return Convert.ToString(Value.ToArgb, 16) 'これだと16進数のみ。

                End Function

                ''' <summary>
                ''' 文字列を色に変換する
                ''' </summary>
                ''' <param name="HexString">16進数文字列</param>
                ''' <returns>《Color》構造体</returns>
                ''' <remarks></remarks>
                Protected Function ColorImport(HexString As String) As Color

                    Dim ret As Color

                    Try
                        If HexString(0) = "#"c Then
                            ret = ColorTranslator.FromHtml(HexString) 'HTML形式(#aarrggbb or #rrggbb)に対応。※上位桁の0を省略しない→#00ff00。
                        Else
                            ret = Color.FromArgb(Convert.ToInt32(HexString, 16)) '《Convert.ToInt32(～,16)》は、"0x"付きでも変換可能。
                        End If
                    Catch ex As Exception
                        ret = Color.FromName(HexString) '16進数として認識できない場合は、色名と捉える。《Color.FromName(～)》には例外が無く、色名として有効でない場合は、透明（ARGB全て0）となる。超便利。
                    End Try

                    'If HexString(0) = "#"c Then
                    '    ret = ColorTranslator.FromHtml(HexString) 'HTML形式(#aarrggbb or #rrggbb)に対応。※上位桁の0を省略しない→#00ff00。
                    'Else
                    '    Dim colorCode As Integer
                    '    HexString = HexString.ToUpper.Replace("&H", "").Replace("0X", "")
                    '    If Integer.TryParse(HexString, Globalization.NumberStyles.HexNumber, Globalization.CultureInfo.CurrentCulture, colorCode) Then '《Integer.TryParse》なら《Try》構文を使わなくて済む。その代わり、"&h","0x"用の工程が増える。
                    '        ret = Color.FromArgb(colorCode)
                    '    Else
                    '        ret = Color.FromName(HexString)
                    '    End If
                    'End If

                    Return ret

                End Function

            End Class



            '============================================================================================================================



            ''' <summary>
            ''' スイッチ付きの色（拡張）
            ''' </summary>
            ''' <remarks>1スイッチ/1色/1線種</remarks>
            <Serializable()>
            Public Class Switch1Color1Style1

                Inherits Switch1Color1

                '----------------------------------------------------------------------------------------------------------------------------

                ''' <summary>
                ''' 線の種類
                ''' </summary>
                <System.Xml.Serialization.XmlElement("線種")>
                Public Property LineStyle() As Drawing2D.DashStyle = Drawing2D.DashStyle.Solid

            End Class



            '============================================================================================================================



            ''' <summary>
            ''' スイッチ付きの色（拡張）
            ''' </summary>
            ''' <remarks>2スイッチ/2色/1模様</remarks>
            <Serializable()>
            Public Class Switch2Color2Texture1

                Inherits Switch1Color1

                '----------------------------------------------------------------------------------------------------------------------------

                ''' <summary>
                ''' 副色
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks>《Color》構造体をシリアル化する方法が分からないのと、16進数ARGB値や色名の方がXMLを直接書き換え易いので、別途《SubColorCode》を用意する。</remarks>
                <System.Xml.Serialization.XmlIgnoreAttribute()>
                Public Property SubColor() As Color = Color.Black

                ''' <summary>
                ''' 副色のコード
                ''' </summary>
                ''' <value>色名（大文字小文字不問）または16進数ARGB値（"0x"付きも可）</value>
                ''' <returns>色名またはARGB値（16進数）</returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("副色")>
                <System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never), System.ComponentModel.Browsable(False)>
                Public Property SubColorCode() As String

                    Get
                        Return ColorExport(SubColor)
                    End Get

                    Set(ByVal value As String)
                        SubColor = ColorImport(value)
                    End Set

                End Property

                ''' <summary>
                ''' 汎用スイッチ
                ''' </summary>
                <System.Xml.Serialization.XmlElement("スイッチ")>
                Public Property Switch As Boolean = True

                ''' <summary>
                ''' 下地の模様
                ''' </summary>
                <System.Xml.Serialization.XmlElement("模様")>
                Public Property Texture() As Drawing2D.HatchStyle = Drawing2D.HatchStyle.Horizontal

            End Class

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' コマンドに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()>
    Public Class CommandParameter

        ''' <summary>
        ''' 色取得用書式に関する設定
        ''' </summary>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("色書式")>
        Public Property ColorFormat As StringTank = New StringTank With {.Ready = New String() {"{3},{4},{5},{6}", "|A16|,|R16|, |G16|, |B16|", "{1:X8}", "{2:X6}", "{6:X2}{5:X2}{4:X2}", "rgba(|R10|, |G10|, |B10|, {7:0.0})"},
                                                                         .Using = "{1:X8}"}

        ''' <summary>
        ''' R,G,B値送信に関する設定
        ''' </summary>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("送信パターン")>
        Public Property SendPattern As StringTank = New StringTank With {.Ready = New String() {"^a|R10|{TAB}^a|G10|{TAB}^a|B10|+{TAB 2}", "^a{4:X}{{TAB}}^a{5:X}{{TAB}}^a{6:X}+{{TAB 2}}", "^a{1:X8}", "^a{1:X8}{{ENTER}}", "^a{2:X6}", "^a{2:X6}{{ENTER}}", "%r|R10|{TAB}|G10|{TAB}|B10|"},
                                                                         .Using = "^a|R10|{TAB}^a|G10|{TAB}^a|B10|+{TAB 2}"}



        '============================================================================================================================



        ''' <summary>
        ''' 文字列貯蔵庫
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()>
        Public Class StringTank

            ''' <summary>
            ''' 既製の文字列
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("既製")>
            Public Property Ready As String()

            ''' <summary>
            ''' 使用中の文字列
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("使用")>
            Public Property [Using] As String

        End Class

    End Class

End Class





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 記録項目
''' </summary>
''' <remarks></remarks>
<Serializable()>
<System.Xml.Serialization.XmlRoot("SCOrPioの記録内容")>
Public Class ResumeParameter

    ''' <summary>
    ''' ビューア寸法
    ''' </summary>
    ''' <returns></returns>
    <System.Xml.Serialization.XmlElement("ビューア寸法")>
    Public Property ViewerSize() As Size = New Size(480, 480)

    ''' <summary>
    ''' 倍率
    ''' </summary>
    ''' <returns></returns>
    <System.Xml.Serialization.XmlElement("倍率")>
    Public Property Magnification() As Integer = 8

    ''' <summary>
    ''' ARGB表示の基数
    ''' </summary>
    ''' <returns>[False]=10進数, [True]=16進数</returns>
    <System.Xml.Serialization.XmlElement("基数")>
    Public Property BaseHex() As Boolean = True

    ''' <summary>
    ''' 基点
    ''' </summary>
    ''' <returns></returns>
    <System.Xml.Serialization.XmlElement("基点")>
    Public Property Cardinal() As Point = Screen.PrimaryScreen.WorkingArea.Location

End Class
